// Fill out your copyright notice in the Description page of Project Settings.

#include "SimpleGame.h"
#include "SimpleWeapon.h"
#include "PublicCharacter.h"


// Sets default values
APublicCharacter::APublicCharacter()
{
 	// Set this character to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

}

USkeletalMeshComponent* APublicCharacter::GetSpecificPawnMesh() const
{
	return GetMesh();
}

FName APublicCharacter::GetWeaponAttachPoint() const
{
	return WeaponAttachPoint;
}

void APublicCharacter::AddWeapon(ASimpleWeapon* Weapon)
{
	if (Weapon)
	{
		Inventory.AddUnique(Weapon);
	}
}

void APublicCharacter::SetCurrentWeapon(class ASimpleWeapon* NewWeapon, class ASimpleWeapon* LastWeapon)
{
	ASimpleWeapon* LocalLastWeapon = NULL;

	if (LastWeapon != NULL)
	{
		LocalLastWeapon = LastWeapon;
	}
	else if (NewWeapon != CurrentWeapon)
	{
		LocalLastWeapon = CurrentWeapon;
	}

	if (LocalLastWeapon)
	{
		LocalLastWeapon->OnUnEquip();
	}

	CurrentWeapon = NewWeapon;

	if (NewWeapon)
	{
		NewWeapon->SetOwningPawn(this);
		NewWeapon->OnEquip(LastWeapon);
	}
}

void APublicCharacter::EquipWeapon(ASimpleWeapon* Weapon)
{
	if (Weapon)
	{
		SetCurrentWeapon(Weapon, CurrentWeapon);
	}
}

void APublicCharacter::SpawnDefaultInventory()
{
	int32 NumWeaponClasses = DefaultInventoryClasses.Num();

	for (int32 i = 0; i < NumWeaponClasses; i++)
	{
		if (DefaultInventoryClasses[i])
		{
			FActorSpawnParameters SpawnInfo;

			ASimpleWeapon* NewWeapon = GetWorld()->SpawnActor<ASimpleWeapon>(DefaultInventoryClasses[i], SpawnInfo);
			AddWeapon(NewWeapon);
		}
	}

	if (Inventory.Num() > 0)
	{
		EquipWeapon(Inventory[0]);
	}
}

void APublicCharacter::OnChangeWeapon()
{
	const int32 CurrentWeaponIndex = Inventory.IndexOfByKey(CurrentWeapon);

	ASimpleWeapon* NextWeapon = Inventory[(CurrentWeaponIndex + 1) % Inventory.Num()];

	EquipWeapon(NextWeapon);
}

void APublicCharacter::WeaponAttack()
{
	if (CurrentWeapon)
	{
		CurrentWeapon->StartAttack();
	}
}

